/* base/src/drv drvVmi4100.c,v 1.6 2004/02/04 18:52:22 mrk Exp */
/***********************************************************************
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory, and the Regents of the University of
* California, as Operator of Los Alamos National Laboratory.
* vmic is distributed subject to a Software License Agreement
* found in file LICENSE that is included with this distribution.
***********************************************************************/
/*
 * Interface to the VMIC 4100 DAC card
 *
 * 	Author:      Bob Dalesio
 * 	Date:        9-26-88
 *
 * Modification Log:
 * -----------------
 * .01	11-09-88	lrd	add LED light status to Ziomek-085
 * .02	02-08-89	lrd	addresses from module_types.h
 *				use 085 structure
 * .03	03-17-89	lrd	add ao_read routine
 * .04	03-29-89	lrd	return correct status on write
 * .05	11-20-89	joh	added call to the at5 vxi driver
 * .06  06-08-90	mrk	fixed bug (R Daly found) for VMI4100
 * .07  10-31-91	 bg	broke vmi4100 driver out of ao_driver.c
 *                              broke vmi4100 code out of io_report and
 *                              created vmi400_io_report()
 * .08  01-10-92	 bg     Added levels to io_report and warning message
 *                              that raw values cannot be read from vmi4100
 *                              card even it the level is 1.
 * .09	08-05-92	joh	arguments to init routine now conform with the
 *				standard
 * .10	08-05-92	joh	added EPICS driver dispatch table	
 * .11	08-05-92	joh	moved parameters from ao_driver.h to here
 * .12	08-11-92	joh	num of cards now dyn configurable	
 * .13	08-24-93	mrk	removed use of variable base_addr
 */

static struct {char ** ppc; char*pc;} RCSID = {&(RCSID.pc),
    "@(#)drvVmi4100.c,v 1.6 2004/02/04 18:52:22 mrk Exp"};

#include <vxWorks.h>
#include <stdlib.h>
#include <stdio.h>
#include <vxLib.h>
#include <sysLib.h>
#include <vme.h>

#include "dbDefs.h"
#include "drvSup.h"
#include <epicsExport.h>

#include "drvVmi4100.h"

static short ao_num_cards[] = {4};
static short ao_num_channels[] = {16};
static unsigned short ao_addrs[] = {0x4100};

/* VMIVME 4100 defines */
#define MAX_AO_VMI_CARDS	(ao_num_cards[VMI4100]) 
#define VMI_MAXCHAN		(ao_num_channels[VMI4100])
#define VMI_ENABLE_OUT		0xc100 /*Fail LED off, enable P3 output.*/

/* memory structure of the Xycom 4100 Interface */

union aoVMI{
        unsigned short csr;
        unsigned short data[16];
};

static long vmi4100_io_report(int level);
static long vmi4100_init(void);

struct {
        long    	number;
        DRVSUPFUN	report;
        DRVSUPFUN       init;
} drvVmi4100={
        2,
        vmi4100_io_report,
        vmi4100_init};
epicsExportAddress(drvet,drvVmi4100);

static unsigned short	**pao_vmi4100;

static int vmi4100_addr;


/*
 * vmi4100_init
 *
 * intialize the VMI analog outputs
 */
static long vmi4100_init(void)
{
	register unsigned short **pcards_present;
	short			shval;
        int                     status;
	register union aoVMI	*pcard;
	register short		i;

	pao_vmi4100 = (unsigned short  **)
		calloc(MAX_AO_VMI_CARDS, sizeof(*pao_vmi4100));
	if(!pao_vmi4100){
		return ERROR;
	}

	pcards_present = pao_vmi4100;

        if ((status = sysBusToLocalAdrs(VME_AM_SUP_SHORT_IO,(char *)(long)ao_addrs[VMI4100], (char **)&vmi4100_addr)) != OK){ 
           printf("Addressing error in vmi4100 driver\n");
           return ERROR;
        }

	pcard = (union aoVMI *)((int)vmi4100_addr);


        /* mark each card present into the card present array */
        for (i = 0; i < ao_num_cards[VMI4100]; i++, pcard+= VMI_MAXCHAN, pcards_present += 1) {

                if (vxMemProbe((char *)pcard,READ,sizeof(short),(char *)&shval) == OK) {
                        *pcards_present = (unsigned short *)pcard;
			pcard->csr = VMI_ENABLE_OUT;
		}
                else{
                        *pcards_present = 0;
                        
                 }
        }

	return OK;
}


/*
 * vmi4100_driver
 *
 * VMI4100 analog output driver
 */
int vmi4100_driver(
    unsigned short card,
    unsigned short chan,
    unsigned short *prval,
    unsigned short *prbval
) {
	register union aoVMI	*paoVMI;

        /* check on the card and channel number as kept in module_types.h */

	if ((paoVMI= (union aoVMI *)pao_vmi4100[card]) == 0)
		return(-1);
	paoVMI->data[chan] = *prval;

	return (0);
}


/*
 * vmi4100_io_report
 *
 * VME analog output driver
 */

static long vmi4100_io_report(int level)
{
    int i;

        for (i = 0; i < MAX_AO_VMI_CARDS; i++){
                if (pao_vmi4100[i]){   
        	        printf("AO: VMI4100:    card %d  ",i);
                	if(level >0){
                      		  printf("VMI4100 card cannot be read.\n");
                        }
                        else 
                                  printf("\n");
                     
                 }
        }

	return OK;
 }

int vmic4100Config(unsigned int ncards, unsigned int nchannels,
    unsigned int base)
{
    ao_num_cards[VMI4100] = ncards;
    ao_num_channels[VMI4100] = nchannels;
    ao_addrs[VMI4100] = base;
    return 0;
}
